==> kotlin-compiler-embeddable-1.1.51-sources.jar: org/jetbrains/kotlin/psi2ir/intermediate/ExtensionInvokeCallReceiver.kt <==
/*
 * Copyright 2010-2017 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jetbrains.kotlin.psi2ir.intermediate

import org.jetbrains.kotlin.ir.expressions.IrExpression

class ExtensionInvokeCallReceiver(
        val callBuilder: CallBuilder,
        val functionReceiver: IntermediateValue,
        val extensionInvokeReceiver: IntermediateValue
) : CallReceiver {
    override fun call(withDispatchAndExtensionReceivers: (IntermediateValue?, IntermediateValue?) -> IrExpression): IrExpression {
        // extensionInvokeReceiver is actually a first argument:
        //      receiver.extFun(p1, ..., pN)
        //      =>
        //      extFun.invoke(receiver, p1, ..., pN)

        assert(callBuilder.irValueArgumentsByIndex[0] == null) {
            "Extension 'invoke' call should have null as its 1st value argument, got: ${callBuilder.irValueArgumentsByIndex[0]}"
        }
        callBuilder.irValueArgumentsByIndex[0] = extensionInvokeReceiver.load()
        return withDispatchAndExtensionReceivers(functionReceiver, null)
    }
}

==> kotlin-compiler-embeddable-1.6.21-sources.jar: org/jetbrains/kotlin/psi2ir/intermediate/ExtensionInvokeCallReceiver.kt <==
/*
 * Copyright 2010-2017 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jetbrains.kotlin.psi2ir.intermediate

import org.jetbrains.kotlin.ir.expressions.IrExpression

class ExtensionInvokeCallReceiver(
    private val callBuilder: CallBuilder,
    private val functionReceiver: IntermediateValue,
    private val extensionInvokeReceiver: IntermediateValue
) : CallReceiver {

    override fun call(withDispatchAndExtensionAndContextReceivers: (IntermediateValue?, IntermediateValue?, List<IntermediateValue>) -> IrExpression): IrExpression {
        // extensionInvokeReceiver is actually a first argument:
        //      receiver.extFun(p1, ..., pN)
        //      =>
        //      extFun.invoke(receiver, p1, ..., pN)

        assert(callBuilder.irValueArgumentsByIndex[0] == null) {
            "Extension 'invoke' call should have null as its 1st value argument, got: ${callBuilder.irValueArgumentsByIndex[0]}"
        }
        callBuilder.irValueArgumentsByIndex[0] = extensionInvokeReceiver.load()
        return withDispatchAndExtensionAndContextReceivers(functionReceiver, null, emptyList())
    }
}
